#!/usr/bin/env python
#
# Copyright 2007 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#



"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database


_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n<google/appengine/api/app_identity/app_identity_service.proto\x12\x10google.appengine\"\xe6\x01\n\x17\x41ppIdentityServiceError\"\xca\x01\n\tErrorCode\x12\x0b\n\x07SUCCESS\x10\x00\x12\x11\n\rUNKNOWN_SCOPE\x10\t\x12\x13\n\x0e\x42LOB_TOO_LARGE\x10\xe8\x07\x12\x16\n\x11\x44\x45\x41\x44LINE_EXCEEDED\x10\xe9\x07\x12\x14\n\x0fNOT_A_VALID_APP\x10\xea\x07\x12\x12\n\rUNKNOWN_ERROR\x10\xeb\x07\x12\x1e\n\x19GAIAMINT_NOT_INITIAILIZED\x10\xec\x07\x12\x10\n\x0bNOT_ALLOWED\x10\xed\x07\x12\x14\n\x0fNOT_IMPLEMENTED\x10\xee\x07\"*\n\x11SignForAppRequest\x12\x15\n\rbytes_to_sign\x18\x01 \x01(\x0c\"?\n\x12SignForAppResponse\x12\x10\n\x08key_name\x18\x01 \x01(\t\x12\x17\n\x0fsignature_bytes\x18\x02 \x01(\x0c\"#\n!GetPublicCertificateForAppRequest\"C\n\x11PublicCertificate\x12\x10\n\x08key_name\x18\x01 \x01(\t\x12\x1c\n\x14x509_certificate_pem\x18\x02 \x01(\t\"\x93\x01\n\"GetPublicCertificateForAppResponse\x12\x44\n\x17public_certificate_list\x18\x01 \x03(\x0b\x32#.google.appengine.PublicCertificate\x12\'\n\x1fmax_client_cache_time_in_second\x18\x02 \x01(\x03\"\x1e\n\x1cGetServiceAccountNameRequest\"=\n\x1dGetServiceAccountNameResponse\x12\x1c\n\x14service_account_name\x18\x01 \x01(\t\"d\n\x15GetAccessTokenRequest\x12\r\n\x05scope\x18\x01 \x03(\t\x12\x1a\n\x12service_account_id\x18\x02 \x01(\x03\x12 \n\x14service_account_name\x18\x03 \x01(\tB\x02\x18\x01\"G\n\x16GetAccessTokenResponse\x12\x14\n\x0c\x61\x63\x63\x65ss_token\x18\x01 \x01(\t\x12\x17\n\x0f\x65xpiration_time\x18\x02 \x01(\x03\" \n\x1eGetDefaultGcsBucketNameRequest\"B\n\x1fGetDefaultGcsBucketNameResponse\x12\x1f\n\x17\x64\x65\x66\x61ult_gcs_bucket_name\x18\x01 \x01(\tB<\n$com.google.appengine.api.appidentityB\x14\x41ppIdentityServicePb')



_APPIDENTITYSERVICEERROR = DESCRIPTOR.message_types_by_name['AppIdentityServiceError']
_SIGNFORAPPREQUEST = DESCRIPTOR.message_types_by_name['SignForAppRequest']
_SIGNFORAPPRESPONSE = DESCRIPTOR.message_types_by_name['SignForAppResponse']
_GETPUBLICCERTIFICATEFORAPPREQUEST = DESCRIPTOR.message_types_by_name['GetPublicCertificateForAppRequest']
_PUBLICCERTIFICATE = DESCRIPTOR.message_types_by_name['PublicCertificate']
_GETPUBLICCERTIFICATEFORAPPRESPONSE = DESCRIPTOR.message_types_by_name['GetPublicCertificateForAppResponse']
_GETSERVICEACCOUNTNAMEREQUEST = DESCRIPTOR.message_types_by_name['GetServiceAccountNameRequest']
_GETSERVICEACCOUNTNAMERESPONSE = DESCRIPTOR.message_types_by_name['GetServiceAccountNameResponse']
_GETACCESSTOKENREQUEST = DESCRIPTOR.message_types_by_name['GetAccessTokenRequest']
_GETACCESSTOKENRESPONSE = DESCRIPTOR.message_types_by_name['GetAccessTokenResponse']
_GETDEFAULTGCSBUCKETNAMEREQUEST = DESCRIPTOR.message_types_by_name['GetDefaultGcsBucketNameRequest']
_GETDEFAULTGCSBUCKETNAMERESPONSE = DESCRIPTOR.message_types_by_name['GetDefaultGcsBucketNameResponse']
_APPIDENTITYSERVICEERROR_ERRORCODE = _APPIDENTITYSERVICEERROR.enum_types_by_name['ErrorCode']
AppIdentityServiceError = _reflection.GeneratedProtocolMessageType('AppIdentityServiceError', (_message.Message,), {
  'DESCRIPTOR' : _APPIDENTITYSERVICEERROR,
  '__module__' : 'google.appengine.api.app_identity.app_identity_service_pb2'

  })
_sym_db.RegisterMessage(AppIdentityServiceError)

SignForAppRequest = _reflection.GeneratedProtocolMessageType('SignForAppRequest', (_message.Message,), {
  'DESCRIPTOR' : _SIGNFORAPPREQUEST,
  '__module__' : 'google.appengine.api.app_identity.app_identity_service_pb2'

  })
_sym_db.RegisterMessage(SignForAppRequest)

SignForAppResponse = _reflection.GeneratedProtocolMessageType('SignForAppResponse', (_message.Message,), {
  'DESCRIPTOR' : _SIGNFORAPPRESPONSE,
  '__module__' : 'google.appengine.api.app_identity.app_identity_service_pb2'

  })
_sym_db.RegisterMessage(SignForAppResponse)

GetPublicCertificateForAppRequest = _reflection.GeneratedProtocolMessageType('GetPublicCertificateForAppRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETPUBLICCERTIFICATEFORAPPREQUEST,
  '__module__' : 'google.appengine.api.app_identity.app_identity_service_pb2'

  })
_sym_db.RegisterMessage(GetPublicCertificateForAppRequest)

PublicCertificate = _reflection.GeneratedProtocolMessageType('PublicCertificate', (_message.Message,), {
  'DESCRIPTOR' : _PUBLICCERTIFICATE,
  '__module__' : 'google.appengine.api.app_identity.app_identity_service_pb2'

  })
_sym_db.RegisterMessage(PublicCertificate)

GetPublicCertificateForAppResponse = _reflection.GeneratedProtocolMessageType('GetPublicCertificateForAppResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETPUBLICCERTIFICATEFORAPPRESPONSE,
  '__module__' : 'google.appengine.api.app_identity.app_identity_service_pb2'

  })
_sym_db.RegisterMessage(GetPublicCertificateForAppResponse)

GetServiceAccountNameRequest = _reflection.GeneratedProtocolMessageType('GetServiceAccountNameRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETSERVICEACCOUNTNAMEREQUEST,
  '__module__' : 'google.appengine.api.app_identity.app_identity_service_pb2'

  })
_sym_db.RegisterMessage(GetServiceAccountNameRequest)

GetServiceAccountNameResponse = _reflection.GeneratedProtocolMessageType('GetServiceAccountNameResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETSERVICEACCOUNTNAMERESPONSE,
  '__module__' : 'google.appengine.api.app_identity.app_identity_service_pb2'

  })
_sym_db.RegisterMessage(GetServiceAccountNameResponse)

GetAccessTokenRequest = _reflection.GeneratedProtocolMessageType('GetAccessTokenRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETACCESSTOKENREQUEST,
  '__module__' : 'google.appengine.api.app_identity.app_identity_service_pb2'

  })
_sym_db.RegisterMessage(GetAccessTokenRequest)

GetAccessTokenResponse = _reflection.GeneratedProtocolMessageType('GetAccessTokenResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETACCESSTOKENRESPONSE,
  '__module__' : 'google.appengine.api.app_identity.app_identity_service_pb2'

  })
_sym_db.RegisterMessage(GetAccessTokenResponse)

GetDefaultGcsBucketNameRequest = _reflection.GeneratedProtocolMessageType('GetDefaultGcsBucketNameRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETDEFAULTGCSBUCKETNAMEREQUEST,
  '__module__' : 'google.appengine.api.app_identity.app_identity_service_pb2'

  })
_sym_db.RegisterMessage(GetDefaultGcsBucketNameRequest)

GetDefaultGcsBucketNameResponse = _reflection.GeneratedProtocolMessageType('GetDefaultGcsBucketNameResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETDEFAULTGCSBUCKETNAMERESPONSE,
  '__module__' : 'google.appengine.api.app_identity.app_identity_service_pb2'

  })
_sym_db.RegisterMessage(GetDefaultGcsBucketNameResponse)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n$com.google.appengine.api.appidentityB\024AppIdentityServicePb'
  _GETACCESSTOKENREQUEST.fields_by_name['service_account_name']._options = None
  _GETACCESSTOKENREQUEST.fields_by_name['service_account_name']._serialized_options = b'\030\001'
  _APPIDENTITYSERVICEERROR._serialized_start=83
  _APPIDENTITYSERVICEERROR._serialized_end=313
  _APPIDENTITYSERVICEERROR_ERRORCODE._serialized_start=111
  _APPIDENTITYSERVICEERROR_ERRORCODE._serialized_end=313
  _SIGNFORAPPREQUEST._serialized_start=315
  _SIGNFORAPPREQUEST._serialized_end=357
  _SIGNFORAPPRESPONSE._serialized_start=359
  _SIGNFORAPPRESPONSE._serialized_end=422
  _GETPUBLICCERTIFICATEFORAPPREQUEST._serialized_start=424
  _GETPUBLICCERTIFICATEFORAPPREQUEST._serialized_end=459
  _PUBLICCERTIFICATE._serialized_start=461
  _PUBLICCERTIFICATE._serialized_end=528
  _GETPUBLICCERTIFICATEFORAPPRESPONSE._serialized_start=531
  _GETPUBLICCERTIFICATEFORAPPRESPONSE._serialized_end=678
  _GETSERVICEACCOUNTNAMEREQUEST._serialized_start=680
  _GETSERVICEACCOUNTNAMEREQUEST._serialized_end=710
  _GETSERVICEACCOUNTNAMERESPONSE._serialized_start=712
  _GETSERVICEACCOUNTNAMERESPONSE._serialized_end=773
  _GETACCESSTOKENREQUEST._serialized_start=775
  _GETACCESSTOKENREQUEST._serialized_end=875
  _GETACCESSTOKENRESPONSE._serialized_start=877
  _GETACCESSTOKENRESPONSE._serialized_end=948
  _GETDEFAULTGCSBUCKETNAMEREQUEST._serialized_start=950
  _GETDEFAULTGCSBUCKETNAMEREQUEST._serialized_end=982
  _GETDEFAULTGCSBUCKETNAMERESPONSE._serialized_start=984
  _GETDEFAULTGCSBUCKETNAMERESPONSE._serialized_end=1050




#!/usr/bin/env python
#
# Copyright 2007 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Implementation of the Cloud Datastore V1 API.

This implementation forwards directly to the v3 service."""













import collections

from google.appengine.api import api_base_pb2
from google.appengine.api import apiproxy_rpc
from google.appengine.api import apiproxy_stub
from google.appengine.api import apiproxy_stub_map
from google.appengine.api import datastore_types
from google.appengine.datastore import cloud_datastore_validator
from google.appengine.datastore import datastore_pb
from google.appengine.datastore import datastore_pbs
from google.appengine.datastore import datastore_query
from google.appengine.datastore import datastore_stub_util
from google.appengine.runtime import apiproxy_errors
from google.appengine.datastore import entity_bytes_pb2 as entity_pb2

_CLOUD_DATASTORE_ENABLED = datastore_pbs._CLOUD_DATASTORE_ENABLED
if _CLOUD_DATASTORE_ENABLED:
  from google.appengine.datastore.datastore_pbs import googledatastore

SERVICE_NAME = 'cloud_datastore_v1'
V3_SERVICE_NAME = 'datastore_v3'


_NO_VERSION = 0

_MINIMUM_VERSION = 1


class _StubIdResolver(datastore_pbs.IdResolver):
  """A IdResolver that converts all project_ids to dev~project_id.

  Users can provide a list of app_ids to override the conversions.
  """

  def __init__(self, app_ids=None):
    """Create a _StubIdResolver.

    Optionally, can provide a list of application ids.
    """
    super(_StubIdResolver, self).__init__(app_ids)

  def resolve_app_id(self, project_id):
    """Resolve the project id. Defaults to dev~project_id."""
    try:
      return super(_StubIdResolver, self).resolve_app_id(project_id)
    except datastore_pbs.InvalidConversionError:
      return 'dev~%s' % project_id


class CloudDatastoreV1Stub(apiproxy_stub.APIProxyStub):
  """Implementation of the Cloud Datastore V1 API.

  This proxies requests to the v3 service."""


  THREADSAFE = False

  def __init__(self, app_id):
    assert _CLOUD_DATASTORE_ENABLED, (
        'Cannot initialize the Cloud Datastore'
        ' stub without installing the Cloud'
        ' Datastore client libraries.')
    apiproxy_stub.APIProxyStub.__init__(self, SERVICE_NAME)
    self.__app_id = app_id
    self._id_resolver = _StubIdResolver([app_id])
    self.__entity_converter = datastore_pbs.get_entity_converter(
        self._id_resolver)
    self.__service_converter = datastore_stub_util.get_service_converter(
        self._id_resolver)
    self.__service_validator = cloud_datastore_validator.get_service_validator(
        self._id_resolver)

  def _Dynamic_BeginTransaction(self, req, resp):


    try:
      self.__service_validator.validate_begin_transaction_req(req)
      v3_req = self.__service_converter.v1_to_v3_begin_transaction_req(
          self.__app_id, req)
    except datastore_pbs.InvalidConversionError as e:
      raise apiproxy_errors.ApplicationError(datastore_pb.Error.BAD_REQUEST,
                                             str(e))
    except cloud_datastore_validator.ValidationError as e:
      raise apiproxy_errors.ApplicationError(datastore_pb.Error.BAD_REQUEST,
                                             str(e))
    v3_resp = datastore_pb.Transaction()
    self.__make_v3_call('BeginTransaction', v3_req, v3_resp)

    try:
      v1_resp = self.__service_converter.v3_to_v1_begin_transaction_resp(
          v3_resp)
    except datastore_pbs.InvalidConversionError as e:
      raise apiproxy_errors.ApplicationError(datastore_pb.Error.INTERNAL_ERROR,
                                             str(e))
    resp.CopyFrom(v1_resp)

  def _Dynamic_Rollback(self, req, unused_resp):


    try:
      self.__service_validator.validate_rollback_req(req)
      v3_req = self.__service_converter.v1_rollback_req_to_v3_txn(req)
    except datastore_pbs.InvalidConversionError as e:
      raise apiproxy_errors.ApplicationError(datastore_pb.Error.BAD_REQUEST,
                                             str(e))
    except cloud_datastore_validator.ValidationError as e:
      raise apiproxy_errors.ApplicationError(datastore_pb.Error.BAD_REQUEST,
                                             str(e))

    self.__make_v3_call('Rollback', v3_req, api_base_pb2.VoidProto())

  def _Dynamic_Commit(self, req, resp):


    try:
      self.__service_validator.validate_commit_req(req)
    except cloud_datastore_validator.ValidationError as e:
      raise apiproxy_errors.ApplicationError(datastore_pb.Error.BAD_REQUEST,
                                             str(e))
    single_use_txn = None
    if req.WhichOneof('transaction_selector') == 'single_use_transaction':
      single_use_txn = self.__begin_adhoc_txn(req)

    try:
      try:
        if req.transaction or single_use_txn:
          self.__commit(req.mutations, req.transaction or single_use_txn, resp)
        else:
          v3_txn_req = datastore_pb.BeginTransactionRequest()
          v3_txn_req.app = self.__app_id

          for mutation in req.mutations:
            v3_txn = datastore_pb.Transaction()
            self.__make_v3_call('BeginTransaction', v3_txn_req, v3_txn)
            v1_txn = self.__service_converter._v3_to_v1_txn(v3_txn)

            commit_resp = googledatastore.CommitResponse()
            self.__commit([mutation], v1_txn, commit_resp)

            resp.index_updates += commit_resp.index_updates
            mutation_result = commit_resp.mutation_results[0]
            resp.mutation_results.add().CopyFrom(mutation_result)
      except datastore_pbs.InvalidConversionError as e:


        raise apiproxy_errors.ApplicationError(datastore_pb.Error.BAD_REQUEST,
                                               str(e))
    except:
      if single_use_txn:
        self.__rollback_adhoc_txn(req, single_use_txn)
      raise

  def _Dynamic_RunQuery(self, req, resp):


    self.__normalize_v1_run_query_request(req)
    snapshot_version = None

    txn = None

    txn_to_cleanup = None

    new_txn = None
    try:
      try:
        self.__service_validator.validate_run_query_req(req)
        if req.read_options.WhichOneof('consistency_type') == 'new_transaction':
          new_txn = self.__begin_adhoc_txn(req)
        v3_req = self.__service_converter.v1_run_query_req_to_v3_query(
            req, new_txn=new_txn)



        if new_txn:
          txn = new_txn
          txn_to_cleanup = new_txn
        elif req.read_options.transaction:
          txn = req.read_options.transaction
        elif (v3_req.HasField('ancestor') and
              req.read_options.read_consistency
              != googledatastore.ReadOptions.EVENTUAL and
              v3_req.kind != '__property__'):
          txn = self.__begin_adhoc_txn(req)
          txn_to_cleanup = txn
          v3_req.transaction.FromString(txn)

      except datastore_pbs.InvalidConversionError as e:
        raise apiproxy_errors.ApplicationError(datastore_pb.Error.BAD_REQUEST,
                                               str(e))
      except cloud_datastore_validator.ValidationError as e:
        raise apiproxy_errors.ApplicationError(datastore_pb.Error.BAD_REQUEST,
                                               str(e))

      v3_resp = datastore_pb.QueryResult()
      self.__make_v3_call('RunQuery', v3_req, v3_resp)



      if txn:
        lookup = googledatastore.LookupRequest()
        lookup.project_id = req.partition_id.project_id
        lookup.database_id = req.partition_id.database_id
        lookup.read_options.transaction = txn
        key = lookup.keys.add()
        key.partition_id.CopyFrom(req.partition_id)
        key.partition_id.database_id = req.database_id
        path = key.path.add()
        path.kind = '__none__'
        path.id = 1
        lookup_response = googledatastore.LookupResponse()
        self._Dynamic_Lookup(lookup, lookup_response)
        snapshot_version = lookup_response.missing[0].version

      try:
        v1_resp = self.__service_converter.v3_to_v1_run_query_resp(
            v3_resp, new_txn=new_txn)
        if req.query.projection:
          if (len(req.query.projection) == 1 and
              req.query.projection[0].property.name == '__key__'):
            result_type = googledatastore.EntityResult.KEY_ONLY
          else:
            result_type = googledatastore.EntityResult.PROJECTION
          v1_resp.batch.entity_result_type = result_type
        if snapshot_version:
          v1_resp.batch.snapshot_version = snapshot_version
      except datastore_pbs.InvalidConversionError as e:
        raise apiproxy_errors.ApplicationError(
            datastore_pb.Error.INTERNAL_ERROR, str(e))
    except:
      if txn_to_cleanup:
        self.__rollback_adhoc_txn(req, txn_to_cleanup)
      raise
    resp.CopyFrom(v1_resp)

  def _Dynamic_Lookup(self, req, resp):


    new_txn = None
    try:
      try:
        self.__service_validator.validate_lookup_req(req)
        if req.read_options.WhichOneof('consistency_type') == 'new_transaction':
          new_txn = self.__begin_adhoc_txn(req)
        v3_req = self.__service_converter.v1_to_v3_get_req(req, new_txn=new_txn)
      except (cloud_datastore_validator.ValidationError,
              datastore_pbs.InvalidConversionError) as e:
        raise apiproxy_errors.ApplicationError(datastore_pb.Error.BAD_REQUEST,
                                               str(e))

      v3_resp = datastore_pb.GetResponse()
      self.__make_v3_call('Get', v3_req, v3_resp)

      try:
        v1_resp = self.__service_converter.v3_to_v1_lookup_resp(v3_resp,
                                                                new_txn=new_txn)
      except datastore_pbs.InvalidConversionError as e:
        raise apiproxy_errors.ApplicationError(datastore_pb.Error.INTERNAL_ERROR,
                                               str(e))
    except:
      if new_txn:
        self.__rollback_adhoc_txn(req, new_txn)
      raise
    resp.CopyFrom(v1_resp)

  def _Dynamic_AllocateIds(self, req, resp):






    v3_stub = apiproxy_stub_map.apiproxy.GetStub(V3_SERVICE_NAME)
    v3_refs = None
    try:
      self.__service_validator.validate_allocate_ids_req(req)
      if req.keys:
        v3_refs = self.__entity_converter.v1_to_v3_references(req.keys)
    except cloud_datastore_validator.ValidationError as e:
      raise apiproxy_errors.ApplicationError(datastore_pb.Error.BAD_REQUEST,
                                             str(e))
    except datastore_pbs.InvalidConversionError as e:
      raise apiproxy_errors.ApplicationError(datastore_pb.Error.BAD_REQUEST,
                                             str(e))
    if v3_refs:
      v3_full_refs = v3_stub._AllocateIds(v3_refs)
      try:
        resp.keys.extend(
            self.__entity_converter.v3_to_v1_keys(v3_full_refs))
      except datastore_pbs.InvalidConversionError as e:
        raise apiproxy_errors.ApplicationError(
            datastore_pb.Error.INTERNAL_ERROR, str(e))

  def __begin_adhoc_txn(self, request):
    """Begins a new transaction as part of another request and returns it.

    Args:
      request: the request that asked for a new transaction to be created.

    Returns:
      a new v1 transaction.
    """
    v1_txn_req = googledatastore.BeginTransactionRequest()
    v1_txn_req.project_id = request.project_id
    v1_txn_resp = googledatastore.BeginTransactionResponse()
    self._Dynamic_BeginTransaction(v1_txn_req, v1_txn_resp)
    return v1_txn_resp.transaction

  def __rollback_adhoc_txn(self, request, v1_transaction):
    """Rolls back a transaction that was created as part of another request.

    This is best effort only, so any error occuring during the rollback will be
    silenced.

    Args:
      request: the request that asked for a new transaction to be created.
      v1_transaction: the transaction that was created and needs to be rolled
          back.
    """
    try:
      v1_rollback_req = googledatastore.RollbackRequest()
      v1_rollback_req.project_id = request.project_id
      v1_rollback_req.transaction = v1_transaction
      self._Dynamic_Rollback(v1_rollback_req,
                             googledatastore.RollbackResponse())
    except apiproxy_errors.ApplicationError:
      pass

  def __commit(self, v1_mutations, v1_txn, resp):
    """Commits a list of v1 mutations.

    Args:
      v1_mutations: the list of mutations to apply and commit
      v1_txn: required v1 transaction handle in which to apply the mutations
      resp: a v1 CommitResponse to update with the result of this commit
    """


    mutation_keys = []


    seen_keys = set()


    allocated_keys = {}


    conflict_cache = {}


    version_cache = {}

    for i, mutation in enumerate(v1_mutations):
      v1_key, v1_entity = datastore_pbs.get_v1_mutation_key_and_entity(mutation)
      key = datastore_types.ReferenceToKeyValue(v1_key, self._id_resolver)
      if not datastore_pbs.is_complete_v1_key(v1_key):

        v1_key = self.__put_v1_entity(v1_entity, v1_txn)
        key = datastore_types.ReferenceToKeyValue(v1_key, self._id_resolver)
        allocated_keys[key] = v1_key
      elif key not in conflict_cache:


        base_version = None
        if mutation.HasField('base_version') and key not in seen_keys:
          base_version = mutation.base_version

        conflict_version = self.__apply_v1_mutation(mutation, base_version,
                                                    v1_txn, version_cache)
        if conflict_version is not None:
          conflict_cache[key] = conflict_version

      mutation_keys.append(key)
      seen_keys.add(key)

    v3_txn = datastore_pb.Transaction()
    self.__service_converter.v1_to_v3_txn(v1_txn, v3_txn)
    v3_resp = datastore_pb.CommitResponse()
    self.__make_v3_call('Commit', v3_txn, v3_resp)
    resp.index_updates = v3_resp.cost.index_writes



    mutation_versions = {}
    for version in v3_resp.version:
      key = datastore_types.ReferenceToKeyValue(version.root_entity_key)
      mutation_versions[key] = version.version

    for key in mutation_keys:
      mutation_result = resp.mutation_results.add()
      if key in allocated_keys:
        mutation_result.key.CopyFrom(allocated_keys[key])
      if key in conflict_cache:
        mutation_result.conflict_detected = True
        mutation_result.version = conflict_cache[key]
      else:
        mutation_result.version = mutation_versions[key]

  def __apply_v1_mutation(self, v1_mutation, base_version, v1_txn,
                          version_cache):
    """Applies a v1 Mutation in a transaction.

    Args:
      v1_mutation: a googledatastore.Mutation, must be for a complete key.
      base_version: optional, the version the entity is expected to be at. If
          the entity has a different version number, the mutation does not
          apply. If None, then this check is skipped.
      v1_txn: a v1 transaction handle
      version_cache: a cache of entity keys to version, for entities that have
          been mutated previously in this transaction.
    """
    v1_key, v1_entity = datastore_pbs.get_v1_mutation_key_and_entity(
        v1_mutation)
    key = datastore_types.ReferenceToKeyValue(v1_key, self._id_resolver)



    if (v1_mutation.HasField('insert') or v1_mutation.HasField('update') or
        base_version is not None) and key not in version_cache:
      version_cache[key] = self.__get_v1_entity_version(v1_key, v1_txn)

    if v1_mutation.HasField('insert'):
      if base_version is not None and base_version != _NO_VERSION:
        raise apiproxy_errors.ApplicationError(datastore_pb.Error.BAD_REQUEST,
                                               'Cannot insert an entity with a '
                                               'base version greater than zero')
      elif version_cache[key] != _NO_VERSION:
        raise apiproxy_errors.ApplicationError(datastore_pb.Error.BAD_REQUEST,
                                               'Entity already exists.')
    elif v1_mutation.HasField('update'):
      if base_version is not None and base_version == _NO_VERSION:
        raise apiproxy_errors.ApplicationError(datastore_pb.Error.BAD_REQUEST,
                                               'Cannot update an entity with a '
                                               'base version set to zero')
      elif version_cache[key] == _NO_VERSION:
        raise apiproxy_errors.ApplicationError(datastore_pb.Error.BAD_REQUEST,
                                               'Entity does not exist.')


    if base_version is not None:
      persisted_version = version_cache[key]
      if persisted_version != _NO_VERSION and persisted_version < base_version:
        raise apiproxy_errors.ApplicationError(datastore_pb.Error.BAD_REQUEST,
                                               'Invalid base version, it is '
                                               'greater than the stored '
                                               'version')
      if persisted_version != base_version:
        return persisted_version


    if v1_mutation.HasField('delete'):
      self.__delete_v1_key(v1_key, v1_txn)
      version_cache[key] = _NO_VERSION
    else:
      self.__put_v1_entity(v1_entity, v1_txn)
      version_cache[key] = _MINIMUM_VERSION

  def __get_v1_entity_version(self, v1_key, v1_txn):
    """Returns the version of an entity, or _NO_VERSION if it does not exist.

    Args:
      v1_key: the key of the entity to lookup.
      v1_txn: the transaction to use when retrieving the entity.

    Returns:
      the version number of the entity if it was found, or _NO_VERSION
      otherwise.
    """
    v3_key = entity_pb2.Reference()
    self.__entity_converter.v1_to_v3_reference(v1_key, v3_key)
    v3_txn = datastore_pb.Transaction()
    self.__service_converter.v1_to_v3_txn(v1_txn, v3_txn)

    v3_get_req = datastore_pb.GetRequest()
    v3_get_req.transaction.CopyFrom(v3_txn)
    v3_get_req.key.append(v3_key)
    v3_get_resp = datastore_pb.GetResponse()
    self.__make_v3_call('Get', v3_get_req, v3_get_resp)
    if v3_get_resp.entity[0].HasField('entity'):
      return v3_get_resp.entity[0].version
    return _NO_VERSION

  def __put_v1_entity(self, v1_entity, v1_txn):
    """Writes a v1 entity to the datastore in a transaction and return its key.

    Args:
      v1_entity: the entity to write
      v1_txn: the transaction in which to write the entity.

    Returns:
      the key of the entity, which may have been allocated.
    """
    v3_entity = entity_pb2.EntityProto()
    self.__entity_converter.v1_to_v3_entity(v1_entity, v3_entity)
    v3_txn = datastore_pb.Transaction()
    self.__service_converter.v1_to_v3_txn(v1_txn, v3_txn)

    v3_put_req = datastore_pb.PutRequest()
    v3_put_req.transaction.CopyFrom(v3_txn)
    v3_put_req.entity.append(v3_entity)
    v3_put_resp = datastore_pb.PutResponse()
    self.__make_v3_call('Put', v3_put_req, v3_put_resp)
    v3_key = v3_put_resp.key[0]

    v1_key = googledatastore.Key()
    self.__entity_converter.v3_to_v1_key(v3_key, v1_key)
    return v1_key

  def __delete_v1_key(self, v1_key, v1_txn):
    """Deletes an entity from a v1 key in a transaction."""
    v3_key = entity_pb2.Reference()
    self.__entity_converter.v1_to_v3_reference(v1_key, v3_key)
    v3_txn = datastore_pb.Transaction()
    self.__service_converter.v1_to_v3_txn(v1_txn, v3_txn)

    v3_delete_req = datastore_pb.DeleteRequest()
    v3_delete_req.transaction.CopyFrom(v3_txn)
    v3_delete_req.key.add().CopyFrom(v3_key)
    v3_delete_resp = datastore_pb.DeleteResponse()
    self.__make_v3_call('Delete', v3_delete_req, v3_delete_resp)

  def __normalize_v1_run_query_request(self, v1_req):

    pass

  def __make_v3_call(self, method, v3_req, v3_resp):
    apiproxy_stub_map.MakeSyncCall(V3_SERVICE_NAME, method, v3_req, v3_resp)



#!/usr/bin/env python
#
# Copyright 2007 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Errors used in the urlfetch API."""











class Error(Exception):
  """Base URL fetcher error type."""


class DownloadError(Error):
  """The URL could not be retrieved.

  This exception is only raised when we cannot contact the server. HTTP errors
  (such as `404`) are returned in the `status_code` field in the return value of
  `fetch`, and no exception is raised.
  """


class MalformedReplyError(DownloadError):
  """The target server returned an invalid HTTP response.

  Responses are invalid if they contain no headers, malformed or incomplete
  headers, or have content missing.
  """


class TooManyRedirectsError(DownloadError):
  """`follow_redirects` was set to `True`, and the redirect limit was hit."""


class InternalTransientError(Error):
  """An internal transient error occurred."""


class ConnectionClosedError(DownloadError):
  """The target server prematurely closed the connection."""


class InvalidURLError(Error):
  """The URL given was empty or invalid.

  Only HTTP and HTTPS URLs are allowed. The maximum URL length is 2048
  characters. The login and password portion is not allowed. In deployed
  applications, only ports `80` and `443` for HTTP and HTTPS respectively are
  allowed.
  """


class PayloadTooLargeError(InvalidURLError):
  """The request payload exceeds the limit."""


class DNSLookupFailedError(DownloadError):
  """The DNS lookup for a URL failed."""


class DeadlineExceededError(DownloadError):
  """The URL was not fetched because the deadline was exceeded.

  This can occur with either the client-supplied `deadline`, or the system
  default if the client does not supply a `deadline` parameter.
  """


class ResponseTooLargeError(Error):
  """The response was too large and was truncated."""

  def __init__(self, response):
    """Constructor."""
    self.response = response


class InvalidMethodError(Error):
  """An invalid value was provided for `method`."""


class SSLCertificateError(Error):
  """An invalid server certificate was presented."""



#!/usr/bin/env python
#
# Copyright 2007 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#



"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database


_sym_db = _symbol_database.Default()


from google.appengine.datastore import entity_v4_pb2 as google_dot_appengine_dot_datastore_dot_entity__v4__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n-google/appengine/datastore/datastore_v4.proto\x12\x1dgoogle.appengine.datastore.v4\x1a*google/appengine/datastore/entity_v4.proto\"\x9f\x03\n\x05\x45rror\"\x95\x03\n\tErrorCode\x12\x0f\n\x0b\x42\x41\x44_REQUEST\x10\x01\x12\x1a\n\x16\x43ONCURRENT_TRANSACTION\x10\x02\x12\x12\n\x0eINTERNAL_ERROR\x10\x03\x12\x0e\n\nNEED_INDEX\x10\x04\x12\x0b\n\x07TIMEOUT\x10\x05\x12\x15\n\x11PERMISSION_DENIED\x10\x06\x12\x12\n\x0e\x42IGTABLE_ERROR\x10\x07\x12 \n\x1c\x43OMMITTED_BUT_STILL_APPLYING\x10\x08\x12\x17\n\x13\x43\x41PABILITY_DISABLED\x10\t\x12\x19\n\x15TRY_ALTERNATE_BACKEND\x10\n\x12\x15\n\x11SAFE_TIME_TOO_OLD\x10\x0b\x12\x16\n\x12RESOURCE_EXHAUSTED\x10\x0c\x12\x1c\n\x18SNAPSHOT_VERSION_TOO_OLD\x10\x12\x12\r\n\tNOT_FOUND\x10\r\x12\x12\n\x0e\x41LREADY_EXISTS\x10\x0e\x12\x17\n\x13\x46\x41ILED_PRECONDITION\x10\x0f\x12\x13\n\x0fUNAUTHENTICATED\x10\x10\x12\x0b\n\x07\x41\x42ORTED\x10\x11\"\x9c\x01\n\x0c\x45ntityResult\x12\x35\n\x06\x65ntity\x18\x01 \x02(\x0b\x32%.google.appengine.datastore.v4.Entity\x12\x0f\n\x07version\x18\x02 \x01(\x03\x12\x0e\n\x06\x63ursor\x18\x03 \x01(\x0c\"4\n\nResultType\x12\x08\n\x04\x46ULL\x10\x01\x12\x0e\n\nPROJECTION\x10\x02\x12\x0c\n\x08KEY_ONLY\x10\x03\"\x8f\x03\n\x05Query\x12\x45\n\nprojection\x18\x02 \x03(\x0b\x32\x31.google.appengine.datastore.v4.PropertyExpression\x12;\n\x04kind\x18\x03 \x03(\x0b\x32-.google.appengine.datastore.v4.KindExpression\x12\x35\n\x06\x66ilter\x18\x04 \x01(\x0b\x32%.google.appengine.datastore.v4.Filter\x12;\n\x05order\x18\x05 \x03(\x0b\x32,.google.appengine.datastore.v4.PropertyOrder\x12\x42\n\x08group_by\x18\x06 \x03(\x0b\x32\x30.google.appengine.datastore.v4.PropertyReference\x12\x14\n\x0cstart_cursor\x18\x07 \x01(\x0c\x12\x12\n\nend_cursor\x18\x08 \x01(\x0c\x12\x11\n\x06offset\x18\n \x01(\x05:\x01\x30\x12\r\n\x05limit\x18\x0b \x01(\x05\"\x1e\n\x0eKindExpression\x12\x0c\n\x04name\x18\x01 \x02(\t\"!\n\x11PropertyReference\x12\x0c\n\x04name\x18\x02 \x02(\t\"\xdf\x01\n\x12PropertyExpression\x12\x42\n\x08property\x18\x01 \x02(\x0b\x32\x30.google.appengine.datastore.v4.PropertyReference\x12\x63\n\x14\x61ggregation_function\x18\x02 \x01(\x0e\x32\x45.google.appengine.datastore.v4.PropertyExpression.AggregationFunction\" \n\x13\x41ggregationFunction\x12\t\n\x05\x46IRST\x10\x01\"\xd5\x01\n\rPropertyOrder\x12\x42\n\x08property\x18\x01 \x02(\x0b\x32\x30.google.appengine.datastore.v4.PropertyReference\x12T\n\tdirection\x18\x02 \x01(\x0e\x32\x36.google.appengine.datastore.v4.PropertyOrder.Direction:\tASCENDING\"*\n\tDirection\x12\r\n\tASCENDING\x10\x01\x12\x0e\n\nDESCENDING\x10\x02\"\x9a\x01\n\x06\x46ilter\x12H\n\x10\x63omposite_filter\x18\x01 \x01(\x0b\x32..google.appengine.datastore.v4.CompositeFilter\x12\x46\n\x0fproperty_filter\x18\x02 \x01(\x0b\x32-.google.appengine.datastore.v4.PropertyFilter\"\xa8\x01\n\x0f\x43ompositeFilter\x12I\n\x08operator\x18\x01 \x02(\x0e\x32\x37.google.appengine.datastore.v4.CompositeFilter.Operator\x12\x35\n\x06\x66ilter\x18\x02 \x03(\x0b\x32%.google.appengine.datastore.v4.Filter\"\x13\n\x08Operator\x12\x07\n\x03\x41ND\x10\x01\"\xd0\x02\n\x0ePropertyFilter\x12\x42\n\x08property\x18\x01 \x02(\x0b\x32\x30.google.appengine.datastore.v4.PropertyReference\x12H\n\x08operator\x18\x02 \x02(\x0e\x32\x36.google.appengine.datastore.v4.PropertyFilter.Operator\x12\x33\n\x05value\x18\x03 \x02(\x0b\x32$.google.appengine.datastore.v4.Value\"{\n\x08Operator\x12\r\n\tLESS_THAN\x10\x01\x12\x16\n\x12LESS_THAN_OR_EQUAL\x10\x02\x12\x10\n\x0cGREATER_THAN\x10\x03\x12\x19\n\x15GREATER_THAN_OR_EQUAL\x10\x04\x12\t\n\x05\x45QUAL\x10\x05\x12\x10\n\x0cHAS_ANCESTOR\x10\x0b\"\xbc\x01\n\x08GqlQuery\x12\x14\n\x0cquery_string\x18\x01 \x02(\t\x12\x1c\n\rallow_literal\x18\x02 \x01(\x08:\x05\x66\x61lse\x12<\n\x08name_arg\x18\x03 \x03(\x0b\x32*.google.appengine.datastore.v4.GqlQueryArg\x12>\n\nnumber_arg\x18\x04 \x03(\x0b\x32*.google.appengine.datastore.v4.GqlQueryArg\"`\n\x0bGqlQueryArg\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x33\n\x05value\x18\x02 \x01(\x0b\x32$.google.appengine.datastore.v4.Value\x12\x0e\n\x06\x63ursor\x18\x03 \x01(\x0c\"\xbb\x03\n\x10QueryResultBatch\x12R\n\x12\x65ntity_result_type\x18\x01 \x02(\x0e\x32\x36.google.appengine.datastore.v4.EntityResult.ResultType\x12\x42\n\rentity_result\x18\x02 \x03(\x0b\x32+.google.appengine.datastore.v4.EntityResult\x12\x16\n\x0eskipped_cursor\x18\x03 \x01(\x0c\x12\x12\n\nend_cursor\x18\x04 \x01(\x0c\x12U\n\x0cmore_results\x18\x05 \x02(\x0e\x32?.google.appengine.datastore.v4.QueryResultBatch.MoreResultsType\x12\x1a\n\x0fskipped_results\x18\x06 \x01(\x05:\x01\x30\x12\x18\n\x10snapshot_version\x18\x07 \x01(\x03\"V\n\x0fMoreResultsType\x12\x10\n\x0cNOT_FINISHED\x10\x01\x12\x1c\n\x18MORE_RESULTS_AFTER_LIMIT\x10\x02\x12\x13\n\x0fNO_MORE_RESULTS\x10\x03\"\x84\x02\n\x08Mutation\x12\x46\n\x02op\x18\x01 \x01(\x0e\x32\x31.google.appengine.datastore.v4.Mutation.Operation:\x07UNKNOWN\x12/\n\x03key\x18\x02 \x01(\x0b\x32\".google.appengine.datastore.v4.Key\x12\x35\n\x06\x65ntity\x18\x03 \x01(\x0b\x32%.google.appengine.datastore.v4.Entity\"H\n\tOperation\x12\x0b\n\x07UNKNOWN\x10\x00\x12\n\n\x06INSERT\x10\x01\x12\n\n\x06UPDATE\x10\x02\x12\n\n\x06UPSERT\x10\x03\x12\n\n\x06\x44\x45LETE\x10\x04\"Y\n\x0eMutationResult\x12/\n\x03key\x18\x03 \x01(\x0b\x32\".google.appengine.datastore.v4.Key\x12\x16\n\x0bnew_version\x18\x04 \x01(\x03:\x01\x30\"\xbb\x02\n\x12\x44\x65precatedMutation\x12\x35\n\x06upsert\x18\x01 \x03(\x0b\x32%.google.appengine.datastore.v4.Entity\x12\x35\n\x06update\x18\x02 \x03(\x0b\x32%.google.appengine.datastore.v4.Entity\x12\x35\n\x06insert\x18\x03 \x03(\x0b\x32%.google.appengine.datastore.v4.Entity\x12=\n\x0einsert_auto_id\x18\x04 \x03(\x0b\x32%.google.appengine.datastore.v4.Entity\x12\x32\n\x06\x64\x65lete\x18\x05 \x03(\x0b\x32\".google.appengine.datastore.v4.Key\x12\r\n\x05\x66orce\x18\x06 \x01(\x08\"\xf1\x01\n\x18\x44\x65precatedMutationResult\x12\x15\n\rindex_updates\x18\x01 \x02(\x05\x12>\n\x12insert_auto_id_key\x18\x02 \x03(\x0b\x32\".google.appengine.datastore.v4.Key\x12\x16\n\x0eupsert_version\x18\x03 \x03(\x03\x12\x16\n\x0eupdate_version\x18\x04 \x03(\x03\x12\x16\n\x0einsert_version\x18\x05 \x03(\x03\x12\x1e\n\x16insert_auto_id_version\x18\x06 \x03(\x03\x12\x16\n\x0e\x64\x65lete_version\x18\x07 \x03(\x03\"\xbb\x01\n\x0bReadOptions\x12]\n\x10read_consistency\x18\x01 \x01(\x0e\x32:.google.appengine.datastore.v4.ReadOptions.ReadConsistency:\x07\x44\x45\x46\x41ULT\x12\x13\n\x0btransaction\x18\x02 \x01(\x0c\"8\n\x0fReadConsistency\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\n\n\x06STRONG\x10\x01\x12\x0c\n\x08\x45VENTUAL\x10\x02\"\x82\x01\n\rLookupRequest\x12@\n\x0cread_options\x18\x01 \x01(\x0b\x32*.google.appengine.datastore.v4.ReadOptions\x12/\n\x03key\x18\x03 \x03(\x0b\x32\".google.appengine.datastore.v4.Key\"\xc0\x01\n\x0eLookupResponse\x12:\n\x05\x66ound\x18\x01 \x03(\x0b\x32+.google.appengine.datastore.v4.EntityResult\x12<\n\x07missing\x18\x02 \x03(\x0b\x32+.google.appengine.datastore.v4.EntityResult\x12\x34\n\x08\x64\x65\x66\x65rred\x18\x03 \x03(\x0b\x32\".google.appengine.datastore.v4.Key\"\xc3\x02\n\x0fRunQueryRequest\x12@\n\x0cread_options\x18\x01 \x01(\x0b\x32*.google.appengine.datastore.v4.ReadOptions\x12@\n\x0cpartition_id\x18\x02 \x01(\x0b\x32*.google.appengine.datastore.v4.PartitionId\x12\x33\n\x05query\x18\x03 \x01(\x0b\x32$.google.appengine.datastore.v4.Query\x12:\n\tgql_query\x18\x07 \x01(\x0b\x32\'.google.appengine.datastore.v4.GqlQuery\x12\x1d\n\x15min_safe_time_seconds\x18\x04 \x01(\x03\x12\x1c\n\x14suggested_batch_size\x18\x05 \x01(\x05\"h\n\x10RunQueryResponse\x12>\n\x05\x62\x61tch\x18\x01 \x02(\x0b\x32/.google.appengine.datastore.v4.QueryResultBatch\x12\x14\n\x0cquery_handle\x18\x02 \x01(\x0c\",\n\x14\x43ontinueQueryRequest\x12\x14\n\x0cquery_handle\x18\x01 \x02(\x0c\"W\n\x15\x43ontinueQueryResponse\x12>\n\x05\x62\x61tch\x18\x01 \x02(\x0b\x32/.google.appengine.datastore.v4.QueryResultBatch\"S\n\x17\x42\x65ginTransactionRequest\x12\x1a\n\x0b\x63ross_group\x18\x01 \x01(\x08:\x05\x66\x61lse\x12\x1c\n\rcross_request\x18\x02 \x01(\x08:\x05\x66\x61lse\"/\n\x18\x42\x65ginTransactionResponse\x12\x13\n\x0btransaction\x18\x01 \x02(\x0c\"&\n\x0fRollbackRequest\x12\x13\n\x0btransaction\x18\x01 \x02(\x0c\"\x12\n\x10RollbackResponse\"\xd2\x02\n\rCommitRequest\x12\x13\n\x0btransaction\x18\x01 \x01(\x0c\x12\x39\n\x08mutation\x18\x05 \x03(\x0b\x32\'.google.appengine.datastore.v4.Mutation\x12N\n\x13\x64\x65precated_mutation\x18\x02 \x01(\x0b\x32\x31.google.appengine.datastore.v4.DeprecatedMutation\x12N\n\x04mode\x18\x04 \x01(\x0e\x32\x31.google.appengine.datastore.v4.CommitRequest.Mode:\rTRANSACTIONAL\x12\x1f\n\x10ignore_read_only\x18\x06 \x01(\x08:\x05\x66\x61lse\"0\n\x04Mode\x12\x11\n\rTRANSACTIONAL\x10\x01\x12\x15\n\x11NON_TRANSACTIONAL\x10\x02\"\xcc\x01\n\x0e\x43ommitResponse\x12\x46\n\x0fmutation_result\x18\x03 \x03(\x0b\x32-.google.appengine.datastore.v4.MutationResult\x12[\n\x1a\x64\x65precated_mutation_result\x18\x01 \x01(\x0b\x32\x37.google.appengine.datastore.v4.DeprecatedMutationResult\x12\x15\n\rindex_updates\x18\x04 \x01(\x05\"\x7f\n\x12\x41llocateIdsRequest\x12\x34\n\x08\x61llocate\x18\x01 \x03(\x0b\x32\".google.appengine.datastore.v4.Key\x12\x33\n\x07reserve\x18\x02 \x03(\x0b\x32\".google.appengine.datastore.v4.Key\"L\n\x13\x41llocateIdsResponse\x12\x35\n\tallocated\x18\x01 \x03(\x0b\x32\".google.appengine.datastore.v4.KeyB\'\n%com.google.google.appengine.datastore')



_ERROR = DESCRIPTOR.message_types_by_name['Error']
_ENTITYRESULT = DESCRIPTOR.message_types_by_name['EntityResult']
_QUERY = DESCRIPTOR.message_types_by_name['Query']
_KINDEXPRESSION = DESCRIPTOR.message_types_by_name['KindExpression']
_PROPERTYREFERENCE = DESCRIPTOR.message_types_by_name['PropertyReference']
_PROPERTYEXPRESSION = DESCRIPTOR.message_types_by_name['PropertyExpression']
_PROPERTYORDER = DESCRIPTOR.message_types_by_name['PropertyOrder']
_FILTER = DESCRIPTOR.message_types_by_name['Filter']
_COMPOSITEFILTER = DESCRIPTOR.message_types_by_name['CompositeFilter']
_PROPERTYFILTER = DESCRIPTOR.message_types_by_name['PropertyFilter']
_GQLQUERY = DESCRIPTOR.message_types_by_name['GqlQuery']
_GQLQUERYARG = DESCRIPTOR.message_types_by_name['GqlQueryArg']
_QUERYRESULTBATCH = DESCRIPTOR.message_types_by_name['QueryResultBatch']
_MUTATION = DESCRIPTOR.message_types_by_name['Mutation']
_MUTATIONRESULT = DESCRIPTOR.message_types_by_name['MutationResult']
_DEPRECATEDMUTATION = DESCRIPTOR.message_types_by_name['DeprecatedMutation']
_DEPRECATEDMUTATIONRESULT = DESCRIPTOR.message_types_by_name['DeprecatedMutationResult']
_READOPTIONS = DESCRIPTOR.message_types_by_name['ReadOptions']
_LOOKUPREQUEST = DESCRIPTOR.message_types_by_name['LookupRequest']
_LOOKUPRESPONSE = DESCRIPTOR.message_types_by_name['LookupResponse']
_RUNQUERYREQUEST = DESCRIPTOR.message_types_by_name['RunQueryRequest']
_RUNQUERYRESPONSE = DESCRIPTOR.message_types_by_name['RunQueryResponse']
_CONTINUEQUERYREQUEST = DESCRIPTOR.message_types_by_name['ContinueQueryRequest']
_CONTINUEQUERYRESPONSE = DESCRIPTOR.message_types_by_name['ContinueQueryResponse']
_BEGINTRANSACTIONREQUEST = DESCRIPTOR.message_types_by_name['BeginTransactionRequest']
_BEGINTRANSACTIONRESPONSE = DESCRIPTOR.message_types_by_name['BeginTransactionResponse']
_ROLLBACKREQUEST = DESCRIPTOR.message_types_by_name['RollbackRequest']
_ROLLBACKRESPONSE = DESCRIPTOR.message_types_by_name['RollbackResponse']
_COMMITREQUEST = DESCRIPTOR.message_types_by_name['CommitRequest']
_COMMITRESPONSE = DESCRIPTOR.message_types_by_name['CommitResponse']
_ALLOCATEIDSREQUEST = DESCRIPTOR.message_types_by_name['AllocateIdsRequest']
_ALLOCATEIDSRESPONSE = DESCRIPTOR.message_types_by_name['AllocateIdsResponse']
_ERROR_ERRORCODE = _ERROR.enum_types_by_name['ErrorCode']
_ENTITYRESULT_RESULTTYPE = _ENTITYRESULT.enum_types_by_name['ResultType']
_PROPERTYEXPRESSION_AGGREGATIONFUNCTION = _PROPERTYEXPRESSION.enum_types_by_name['AggregationFunction']
_PROPERTYORDER_DIRECTION = _PROPERTYORDER.enum_types_by_name['Direction']
_COMPOSITEFILTER_OPERATOR = _COMPOSITEFILTER.enum_types_by_name['Operator']
_PROPERTYFILTER_OPERATOR = _PROPERTYFILTER.enum_types_by_name['Operator']
_QUERYRESULTBATCH_MORERESULTSTYPE = _QUERYRESULTBATCH.enum_types_by_name['MoreResultsType']
_MUTATION_OPERATION = _MUTATION.enum_types_by_name['Operation']
_READOPTIONS_READCONSISTENCY = _READOPTIONS.enum_types_by_name['ReadConsistency']
_COMMITREQUEST_MODE = _COMMITREQUEST.enum_types_by_name['Mode']
Error = _reflection.GeneratedProtocolMessageType('Error', (_message.Message,), {
  'DESCRIPTOR' : _ERROR,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(Error)

EntityResult = _reflection.GeneratedProtocolMessageType('EntityResult', (_message.Message,), {
  'DESCRIPTOR' : _ENTITYRESULT,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(EntityResult)

Query = _reflection.GeneratedProtocolMessageType('Query', (_message.Message,), {
  'DESCRIPTOR' : _QUERY,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(Query)

KindExpression = _reflection.GeneratedProtocolMessageType('KindExpression', (_message.Message,), {
  'DESCRIPTOR' : _KINDEXPRESSION,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(KindExpression)

PropertyReference = _reflection.GeneratedProtocolMessageType('PropertyReference', (_message.Message,), {
  'DESCRIPTOR' : _PROPERTYREFERENCE,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(PropertyReference)

PropertyExpression = _reflection.GeneratedProtocolMessageType('PropertyExpression', (_message.Message,), {
  'DESCRIPTOR' : _PROPERTYEXPRESSION,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(PropertyExpression)

PropertyOrder = _reflection.GeneratedProtocolMessageType('PropertyOrder', (_message.Message,), {
  'DESCRIPTOR' : _PROPERTYORDER,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(PropertyOrder)

Filter = _reflection.GeneratedProtocolMessageType('Filter', (_message.Message,), {
  'DESCRIPTOR' : _FILTER,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(Filter)

CompositeFilter = _reflection.GeneratedProtocolMessageType('CompositeFilter', (_message.Message,), {
  'DESCRIPTOR' : _COMPOSITEFILTER,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(CompositeFilter)

PropertyFilter = _reflection.GeneratedProtocolMessageType('PropertyFilter', (_message.Message,), {
  'DESCRIPTOR' : _PROPERTYFILTER,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(PropertyFilter)

GqlQuery = _reflection.GeneratedProtocolMessageType('GqlQuery', (_message.Message,), {
  'DESCRIPTOR' : _GQLQUERY,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(GqlQuery)

GqlQueryArg = _reflection.GeneratedProtocolMessageType('GqlQueryArg', (_message.Message,), {
  'DESCRIPTOR' : _GQLQUERYARG,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(GqlQueryArg)

QueryResultBatch = _reflection.GeneratedProtocolMessageType('QueryResultBatch', (_message.Message,), {
  'DESCRIPTOR' : _QUERYRESULTBATCH,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(QueryResultBatch)

Mutation = _reflection.GeneratedProtocolMessageType('Mutation', (_message.Message,), {
  'DESCRIPTOR' : _MUTATION,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(Mutation)

MutationResult = _reflection.GeneratedProtocolMessageType('MutationResult', (_message.Message,), {
  'DESCRIPTOR' : _MUTATIONRESULT,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(MutationResult)

DeprecatedMutation = _reflection.GeneratedProtocolMessageType('DeprecatedMutation', (_message.Message,), {
  'DESCRIPTOR' : _DEPRECATEDMUTATION,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(DeprecatedMutation)

DeprecatedMutationResult = _reflection.GeneratedProtocolMessageType('DeprecatedMutationResult', (_message.Message,), {
  'DESCRIPTOR' : _DEPRECATEDMUTATIONRESULT,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(DeprecatedMutationResult)

ReadOptions = _reflection.GeneratedProtocolMessageType('ReadOptions', (_message.Message,), {
  'DESCRIPTOR' : _READOPTIONS,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(ReadOptions)

LookupRequest = _reflection.GeneratedProtocolMessageType('LookupRequest', (_message.Message,), {
  'DESCRIPTOR' : _LOOKUPREQUEST,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(LookupRequest)

LookupResponse = _reflection.GeneratedProtocolMessageType('LookupResponse', (_message.Message,), {
  'DESCRIPTOR' : _LOOKUPRESPONSE,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(LookupResponse)

RunQueryRequest = _reflection.GeneratedProtocolMessageType('RunQueryRequest', (_message.Message,), {
  'DESCRIPTOR' : _RUNQUERYREQUEST,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(RunQueryRequest)

RunQueryResponse = _reflection.GeneratedProtocolMessageType('RunQueryResponse', (_message.Message,), {
  'DESCRIPTOR' : _RUNQUERYRESPONSE,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(RunQueryResponse)

ContinueQueryRequest = _reflection.GeneratedProtocolMessageType('ContinueQueryRequest', (_message.Message,), {
  'DESCRIPTOR' : _CONTINUEQUERYREQUEST,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(ContinueQueryRequest)

ContinueQueryResponse = _reflection.GeneratedProtocolMessageType('ContinueQueryResponse', (_message.Message,), {
  'DESCRIPTOR' : _CONTINUEQUERYRESPONSE,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(ContinueQueryResponse)

BeginTransactionRequest = _reflection.GeneratedProtocolMessageType('BeginTransactionRequest', (_message.Message,), {
  'DESCRIPTOR' : _BEGINTRANSACTIONREQUEST,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(BeginTransactionRequest)

BeginTransactionResponse = _reflection.GeneratedProtocolMessageType('BeginTransactionResponse', (_message.Message,), {
  'DESCRIPTOR' : _BEGINTRANSACTIONRESPONSE,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(BeginTransactionResponse)

RollbackRequest = _reflection.GeneratedProtocolMessageType('RollbackRequest', (_message.Message,), {
  'DESCRIPTOR' : _ROLLBACKREQUEST,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(RollbackRequest)

RollbackResponse = _reflection.GeneratedProtocolMessageType('RollbackResponse', (_message.Message,), {
  'DESCRIPTOR' : _ROLLBACKRESPONSE,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(RollbackResponse)

CommitRequest = _reflection.GeneratedProtocolMessageType('CommitRequest', (_message.Message,), {
  'DESCRIPTOR' : _COMMITREQUEST,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(CommitRequest)

CommitResponse = _reflection.GeneratedProtocolMessageType('CommitResponse', (_message.Message,), {
  'DESCRIPTOR' : _COMMITRESPONSE,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(CommitResponse)

AllocateIdsRequest = _reflection.GeneratedProtocolMessageType('AllocateIdsRequest', (_message.Message,), {
  'DESCRIPTOR' : _ALLOCATEIDSREQUEST,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(AllocateIdsRequest)

AllocateIdsResponse = _reflection.GeneratedProtocolMessageType('AllocateIdsResponse', (_message.Message,), {
  'DESCRIPTOR' : _ALLOCATEIDSRESPONSE,
  '__module__' : 'google.appengine.datastore.datastore_v4_pb2'

  })
_sym_db.RegisterMessage(AllocateIdsResponse)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n%com.google.google.appengine.datastore'
  _ERROR._serialized_start=125
  _ERROR._serialized_end=540
  _ERROR_ERRORCODE._serialized_start=135
  _ERROR_ERRORCODE._serialized_end=540
  _ENTITYRESULT._serialized_start=543
  _ENTITYRESULT._serialized_end=699
  _ENTITYRESULT_RESULTTYPE._serialized_start=647
  _ENTITYRESULT_RESULTTYPE._serialized_end=699
  _QUERY._serialized_start=702
  _QUERY._serialized_end=1101
  _KINDEXPRESSION._serialized_start=1103
  _KINDEXPRESSION._serialized_end=1133
  _PROPERTYREFERENCE._serialized_start=1135
  _PROPERTYREFERENCE._serialized_end=1168
  _PROPERTYEXPRESSION._serialized_start=1171
  _PROPERTYEXPRESSION._serialized_end=1394
  _PROPERTYEXPRESSION_AGGREGATIONFUNCTION._serialized_start=1362
  _PROPERTYEXPRESSION_AGGREGATIONFUNCTION._serialized_end=1394
  _PROPERTYORDER._serialized_start=1397
  _PROPERTYORDER._serialized_end=1610
  _PROPERTYORDER_DIRECTION._serialized_start=1568
  _PROPERTYORDER_DIRECTION._serialized_end=1610
  _FILTER._serialized_start=1613
  _FILTER._serialized_end=1767
  _COMPOSITEFILTER._serialized_start=1770
  _COMPOSITEFILTER._serialized_end=1938
  _COMPOSITEFILTER_OPERATOR._serialized_start=1919
  _COMPOSITEFILTER_OPERATOR._serialized_end=1938
  _PROPERTYFILTER._serialized_start=1941
  _PROPERTYFILTER._serialized_end=2277
  _PROPERTYFILTER_OPERATOR._serialized_start=2154
  _PROPERTYFILTER_OPERATOR._serialized_end=2277
  _GQLQUERY._serialized_start=2280
  _GQLQUERY._serialized_end=2468
  _GQLQUERYARG._serialized_start=2470
  _GQLQUERYARG._serialized_end=2566
  _QUERYRESULTBATCH._serialized_start=2569
  _QUERYRESULTBATCH._serialized_end=3012
  _QUERYRESULTBATCH_MORERESULTSTYPE._serialized_start=2926
  _QUERYRESULTBATCH_MORERESULTSTYPE._serialized_end=3012
  _MUTATION._serialized_start=3015
  _MUTATION._serialized_end=3275
  _MUTATION_OPERATION._serialized_start=3203
  _MUTATION_OPERATION._serialized_end=3275
  _MUTATIONRESULT._serialized_start=3277
  _MUTATIONRESULT._serialized_end=3366
  _DEPRECATEDMUTATION._serialized_start=3369
  _DEPRECATEDMUTATION._serialized_end=3684
  _DEPRECATEDMUTATIONRESULT._serialized_start=3687
  _DEPRECATEDMUTATIONRESULT._serialized_end=3928
  _READOPTIONS._serialized_start=3931
  _READOPTIONS._serialized_end=4118
  _READOPTIONS_READCONSISTENCY._serialized_start=4062
  _READOPTIONS_READCONSISTENCY._serialized_end=4118
  _LOOKUPREQUEST._serialized_start=4121
  _LOOKUPREQUEST._serialized_end=4251
  _LOOKUPRESPONSE._serialized_start=4254
  _LOOKUPRESPONSE._serialized_end=4446
  _RUNQUERYREQUEST._serialized_start=4449
  _RUNQUERYREQUEST._serialized_end=4772
  _RUNQUERYRESPONSE._serialized_start=4774
  _RUNQUERYRESPONSE._serialized_end=4878
  _CONTINUEQUERYREQUEST._serialized_start=4880
  _CONTINUEQUERYREQUEST._serialized_end=4924
  _CONTINUEQUERYRESPONSE._serialized_start=4926
  _CONTINUEQUERYRESPONSE._serialized_end=5013
  _BEGINTRANSACTIONREQUEST._serialized_start=5015
  _BEGINTRANSACTIONREQUEST._serialized_end=5098
  _BEGINTRANSACTIONRESPONSE._serialized_start=5100
  _BEGINTRANSACTIONRESPONSE._serialized_end=5147
  _ROLLBACKREQUEST._serialized_start=5149
  _ROLLBACKREQUEST._serialized_end=5187
  _ROLLBACKRESPONSE._serialized_start=5189
  _ROLLBACKRESPONSE._serialized_end=5207
  _COMMITREQUEST._serialized_start=5210
  _COMMITREQUEST._serialized_end=5548
  _COMMITREQUEST_MODE._serialized_start=5500
  _COMMITREQUEST_MODE._serialized_end=5548
  _COMMITRESPONSE._serialized_start=5551
  _COMMITRESPONSE._serialized_end=5755
  _ALLOCATEIDSREQUEST._serialized_start=5757
  _ALLOCATEIDSREQUEST._serialized_end=5884
  _ALLOCATEIDSRESPONSE._serialized_start=5886
  _ALLOCATEIDSRESPONSE._serialized_end=5962




#!/usr/bin/env python
#
# Copyright 2007 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#














"""Test utilities for writing NDB tests.

Useful set of utilities for correctly setting up the appengine testing
environment.  Functions and test-case base classes that configure stubs
and other environment variables.
"""

import logging
import os
import shutil
import tempfile
import types

from google.appengine.ext.ndb import context
from google.appengine.ext.ndb import eventloop
from google.appengine.ext.ndb import model
from google.appengine.ext.ndb import tasklets

from six.moves import range

from google.appengine.api import apiproxy_stub_map
from google.appengine.datastore import cloud_datastore_v1_remote_stub
from google.appengine.datastore import datastore_pbs
from google.appengine.datastore import datastore_rpc
from google.appengine.datastore import datastore_stub_util
from google.appengine.ext import testbed
from absl.testing import absltest as unittest


class NDBBaseTest(unittest.TestCase):
  """Base class for tests that interact with API stubs or create Models.

  NOTE: Care must be used when working with model classes using this test
  class.  The kind-map is reset on each iteration.  The general practice
  should be to declare test models in the sub-classes setUp method AFTER
  calling this classes setUp method.
  """

  APP_ID = 'ndb-test-app-id'

  def setUp(self):
    """Set up test framework.

    Configures basic environment variables, stubs and creates a default
    connection.
    """
    super(NDBBaseTest, self).setUp()

    self.testbed = testbed.Testbed()
    self.testbed.setup_env(overwrite=True, app_id=self.APP_ID)
    self.testbed.activate()

    self.ResetKindMap()
    self.conn = self.MakeConnection()
    self.ctx = self.MakeContext(conn=self.conn)
    tasklets.set_context(self.ctx)

    self._logger = logging.getLogger()
    self._old_log_level = self._logger.getEffectiveLevel()

  def ExpectErrors(self):
    if self.DefaultLogging():
      self._logger.setLevel(logging.CRITICAL)

  def ExpectWarnings(self):
    if self.DefaultLogging():
      self._logger.setLevel(logging.ERROR)

  def DefaultLogging(self):
    return self._old_log_level == logging.WARNING

  def tearDown(self):
    """Tear down test framework."""
    super(NDBBaseTest, self).tearDown()

    self.testbed.deactivate()
    self._logger.setLevel(self._old_log_level)
    ev = eventloop.get_event_loop()
    stragglers = 0
    while ev.run1():
      stragglers += 1
    if stragglers:
      logging.info('Processed %d straggler events after test completed',
                   stragglers)
    self.ResetKindMap()

  def ResetKindMap(self):
    model.Model._reset_kind_map()

  def MakeContext(self, config=None, auto_batcher_class=context.AutoBatcher,
                  default_model=None, conn=None):
    if not conn:
      conn = self.MakeConnection(config=config,
                                 default_model=default_model)
    ctx = context.Context(
        conn=conn,
        auto_batcher_class=auto_batcher_class,
        config=config)



    ctx.set_cache_policy(False)
    ctx.set_memcache_policy(False)
    return ctx


class NDBTest(NDBBaseTest):


  the_module = None

  def testAllVariableIsConsistent(self):

    if self.the_module is None:
      return

    modname = self.the_module.__name__
    undefined = []
    for name in self.the_module.__all__:
      if not hasattr(self.the_module, name):
        undefined.append(name)
    self.assertFalse(undefined,
                     '%s.__all__ has some names that are not defined: %s' %
                     (modname, undefined))

    ignored_types = (types.ModuleType, type(range))

    unlisted = []
    for name in dir(self.the_module):
      if not name.startswith('_'):
        obj = getattr(self.the_module, name)
        if not isinstance(obj, ignored_types):
          if name not in self.the_module.__all__:
            unlisted.append(name)
    self.assertFalse(unlisted,
                     '%s defines some names that are not in __all__: %s' %
                     (modname, unlisted))

  def HRTest(self):
    ds_stub = self.testbed.get_stub('datastore_v3')
    hrd_policy = datastore_stub_util.PseudoRandomHRConsistencyPolicy(
        probability=1)
    ds_stub.SetConsistencyPolicy(hrd_policy)

  def setUp(self):
    super(NDBTest, self).setUp()




    self.app_root_path = tempfile.mkdtemp()

    self.testbed.init_datastore_v3_stub(root_path=self.app_root_path)
    self.testbed.init_memcache_stub()
    self.testbed.init_taskqueue_stub()

  def tearDown(self):
    super(NDBTest, self).tearDown()
    shutil.rmtree(self.app_root_path)

  def MakeConnection(self, *args, **kwargs):
    return model.make_connection(*args, **kwargs)


class NDBCloudDatastoreV1Test(NDBBaseTest):
  """NDB test base that uses a datastore V1 connection."""

  @classmethod
  def setUpClass(cls):
    super(NDBCloudDatastoreV1Test, cls).setUpClass()



    from google.appengine.ext.ndb import datastore_emulator
    factory = datastore_emulator.DatastoreEmulatorFactory()
    cls.datastore = factory.Create(cls.APP_ID)

  @classmethod
  def tearDownClass(cls):
    super(NDBCloudDatastoreV1Test, cls).tearDownClass()
    cls.datastore.Stop()

  def setUp(self):
    super(NDBCloudDatastoreV1Test, self).setUp()



    os.environ['DATASTORE_EMULATOR_HOST'] = 'localhost:1234'
    self.datastore.Clear()
    stub = cloud_datastore_v1_remote_stub.CloudDatastoreV1RemoteStub(
        self.datastore.GetDatastore())
    apiproxy_stub_map.apiproxy.ReplaceStub(datastore_rpc._CLOUD_DATASTORE_V1,
                                           stub)

  def tearDown(self):
    super(NDBCloudDatastoreV1Test, self).tearDown()
    self.datastore.Clear()

  def HRTest(self):
    pass

  def MakeConnection(self, *args, **kwargs):
    if '_api_version' not in kwargs:
      kwargs['_api_version'] = datastore_rpc._CLOUD_DATASTORE_V1
    if '_id_resolver' not in kwargs:
      kwargs['_id_resolver'] = datastore_pbs.IdResolver([self.APP_ID])
    return model.make_connection(*args, **kwargs)



#!/usr/bin/env python
#
# Copyright 2007 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""Test func/class decorator that runs tests in a clean context.

Usage:

@ctx_test_util.isolated_context()
class MySuite(unittest.TestCase):
  ...

or

class MySuite(unittest.TestCase):

  @ctx_test_util.isolated_context()
  def testFoo(self):
    ...

"""

import contextvars
import functools
import os

from google.appengine.runtime import context


def isolated_context(*args, **kwargs):
  """Decorates a test function or class to run itself in a clean context.

  When used with a class, it will run each test function in its own context.
  It will also run setUp, tearDown, and doCleanups in that context by default.

  Args:
    *args: See _isolated_context_class and _isolated_context_callable.
    **kwargs: See _isolated_context_class and _isolated_context_callable.

  Returns:
    The raw function decorator.
  """
  def argless_decorator(func_or_class):
    if isinstance(func_or_class, type):
      return _isolated_context_class(*args, **kwargs)(func_or_class)
    return _isolated_context_callable(*args, **kwargs)(func_or_class)
  return argless_decorator


def both_context_modes(*args, **kwargs):
  """Adds subtests which test the non-default context mode.

  This decorator also calls isolated_context for you.
  Using this as a class decorator will only wrap the functions declared directly
  on that class, not on functions inherited from parent classes.

  Args:
    *args: See _isolated_context_class and _isolated_context_callable
    **kwargs: See _isolated_context_class and _isolated_context_callable

  Returns:
    The raw function decorator.
  """
  def argless_decorator(func_or_class):
    if isinstance(func_or_class, type):
      return _both_context_modes_class(*args, **kwargs)(func_or_class)
    return _both_context_modes_callable(*args, **kwargs)(func_or_class)
  return argless_decorator


def _isolated_context_callable(
    backup_and_restore_os_environ=context.READ_FROM_OS_ENVIRON):
  """Decorates a function to run itself in a clean context."""

  def argless_decorator(func):
    def run_in_ctx(*args, **kwargs):
      contextvars.Context().run(func, *args, **kwargs)
    if backup_and_restore_os_environ:
      run_in_ctx = _isolate_os_environ(run_in_ctx)
    return run_in_ctx

  return argless_decorator


def _isolate_os_environ(func):
  """Save and restore os.environ."""

  @functools.wraps(func)
  def run_in_isolated_os_environ(*args, **kwargs):
    previous_os_environ = os.environ.copy()
    try:
      func(*args, **kwargs)
    finally:
      os.environ.clear()
      os.environ.update(previous_os_environ)

  return run_in_isolated_os_environ


def _isolated_context_class(
    backup_and_restore_os_environ=context.READ_FROM_OS_ENVIRON,
    run_setup_teardown_cleanup_inside_context=True):
  """Decorates a test class to run itself in a clean context."""

  isolate_fn_wrapper = _isolated_context_callable(backup_and_restore_os_environ)

  def argless_decorator(klass):
    if run_setup_teardown_cleanup_inside_context:
      run_fn = getattr(
          klass, 'run',
          lambda self, *args, **kwargs: super(klass, self).run(*args, **kwargs))
      setattr(klass, 'run', isolate_fn_wrapper(run_fn))
    else:
      for func_name, func in vars(klass).items():
        if callable(func) and func_name.startswith('test'):
          setattr(klass, func_name, isolate_fn_wrapper(func))
    return klass
  return argless_decorator


def _both_context_modes_class():
  """Class decorator that duplicates every test function each context mode."""

  def argless_decorator(klass):
    klass = _isolated_context_class(backup_and_restore_os_environ=True)(klass)
    for func_name, func in vars(klass).items():
      if callable(func) and func_name.startswith('test'):
        setattr(klass, func_name, _both_context_modes_callable_raw(func))
    return klass

  return argless_decorator


def _both_context_modes_callable():
  def argless_decorator(func):
    return _isolated_context_callable()(_both_context_modes_callable_raw(func))
  return argless_decorator


def set_both(key, value):
  """Write to both legacy context (os.environ) and new contextvars."""


  if key == 'DEFAULT_NAMESPACE' or key == 'HTTP_X_APPENGINE_DEFAULT_NAMESPACE':
    key = 'DEFAULT_NAMESPACE'
    os_env_key = 'HTTP_X_APPENGINE_DEFAULT_NAMESPACE'
  else:
    os_env_key = key



  strval = value
  if key == 'USER_IS_ADMIN':

    if isinstance(value, str):
      value = value == '1'
    elif isinstance(value, bool):
      strval = '1' if value else '0'
  os.environ[os_env_key] = strval
  ctxvar = vars(context.gae_headers).get(key, vars(context.wsgi).get(key))
  assert isinstance(ctxvar, contextvars.ContextVar)
  ctxvar.set(value)


def _both_context_modes_callable_raw(func):
  """Tests both context modes WITHOUT isolating context.

  This is factored out of _both_context_modes_callable so that
  @both_context_modes doesn't double-isolate functions when used as a class
  decorator.

  Args:
    func: The function to wrap.

  Returns:
    The wrapped function.
  """
  @functools.wraps(func)
  def test_both(self, *args, **kwargs):
    default_context_mode = context.READ_FROM_OS_ENVIRON
    other_context_mode = not default_context_mode
    try:
      func(self, *args, **kwargs)
      context.READ_FROM_OS_ENVIRON = other_context_mode
      with self.subTest(READ_CONTEXT_FROM_OS_ENVIRON=other_context_mode):
        func(self, *args, **kwargs)
    finally:
      context.READ_FROM_OS_ENVIRON = default_context_mode
  return test_both



#!/usr/bin/env python
#
# Copyright 2007 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
















"""NDB interface for Blobstore.

This currently builds on google.appengine.ext.blobstore and provides a
similar API.  The main API differences:

- BlobInfo is an actual Model subclass rather than a pseudo-model class.
  To query, use BlobInfo.query() and its documented properties.  Other
  changes:
  - The kind is '__BlobInfo__' (BLOB_INFO_KIND).
  - key() is a method returning a BlobKey instance.
  - put() and friends are disabled.
  - Added class methods get() and friends.
  - Added instance methods delete() and friends, and open().

- Instead of BlobReferenceProperty, there's BlobKeyProperty.

- There is no create_rpc() function.  Instead, functions and methods
  take keyword arguments to specify deadline, callback, and (in some
  case) datastore options.

- APIs (get() and delete()) that in ext.blobstore take either a blob
  key or a list of blob keys are split into two: one that takes a blob
  key and one that takes a list of blob keys, the latter having a name
  ending in _multi.

- The following APIs have a synchronous and an async version:
  - BlobInfo.get()
  - BlobInfo.delete()
  - create_upload_url()
  - get()
  - get_multi()
  - delete()
  - delete_multi()
  - fetch_data()
"""





from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import base64
import email

from google.appengine.ext.ndb import model
from google.appengine.ext.ndb import tasklets
import six
from six.moves import map

from google.appengine.api.blobstore import blobstore
from google.appengine.ext.blobstore import blobstore as ext_blobstore

__all__ = ['BLOB_INFO_KIND',
           'BLOB_KEY_HEADER',
           'BLOB_MIGRATION_KIND',
           'BLOB_RANGE_HEADER',
           'BlobFetchSizeTooLargeError',
           'BlobInfo',
           'BlobInfoParseError',
           'BlobKey',
           'BlobNotFoundError',
           'BlobKeyProperty',
           'BlobReader',
           'DataIndexOutOfRangeError',
           'PermissionDeniedError',
           'Error',
           'InternalError',
           'MAX_BLOB_FETCH_SIZE',
           'UPLOAD_INFO_CREATION_HEADER',
           'create_upload_url',
           'create_upload_url_async',
           'delete',
           'delete_async',
           'delete_multi',
           'delete_multi_async',
           'fetch_data',
           'fetch_data_async',
           'get',
           'get_async',
           'get_multi',
           'get_multi_async',
           'parse_blob_info']


Error = blobstore.Error
InternalError = blobstore.InternalError
BlobFetchSizeTooLargeError = blobstore.BlobFetchSizeTooLargeError
BlobNotFoundError = blobstore.BlobNotFoundError
_CreationFormatError = blobstore._CreationFormatError
DataIndexOutOfRangeError = blobstore.DataIndexOutOfRangeError
PermissionDeniedError = blobstore.PermissionDeniedError
BlobInfoParseError = ext_blobstore.BlobInfoParseError


BlobKey = blobstore.BlobKey


BLOB_INFO_KIND = blobstore.BLOB_INFO_KIND
BLOB_MIGRATION_KIND = blobstore.BLOB_MIGRATION_KIND
BLOB_KEY_HEADER = blobstore.BLOB_KEY_HEADER
BLOB_RANGE_HEADER = blobstore.BLOB_RANGE_HEADER
MAX_BLOB_FETCH_SIZE = blobstore.MAX_BLOB_FETCH_SIZE
UPLOAD_INFO_CREATION_HEADER = blobstore.UPLOAD_INFO_CREATION_HEADER


BlobKeyProperty = model.BlobKeyProperty


class BlobInfo(model.Model):
  """Information about blobs in Blobstore.

  This is a Model subclass that has been doctored to be unwritable.

  Properties:
  - content_type: Content type of blob.
  - creation: Creation date of blob, when it was uploaded.
  - filename: Filename user selected from their machine.
  - size: Size of uncompressed blob.
  - md5_hash: The md5 hash value of the uploaded blob (in hex).

  Additional API:

  Class methods:
  - get(): retrieve a BlobInfo by key
  - get_multi(): retrieve a list of BlobInfos by keys
  - get_async(), get_multi_async(): async version of get() and get_multi()

  Instance methods:
  - delete(): delete this blob
  - delete_async(): async version of delete()
  - key(): return the BlobKey for this blob
  - open(): return a BlobReader instance for this blob

  Because BlobInfo instances are synchronized with Blobstore, the class
  cache policies are off.

  Do not subclass this class.
  """

  _use_cache = False
  _use_memcache = False

  content_type = model.StringProperty()
  creation = model.DateTimeProperty()
  filename = model.StringProperty()
  size = model.IntegerProperty()
  md5_hash = model.StringProperty()

  @classmethod
  def _get_kind(cls):
    """Override this to match the datastore entities written by Blobstore."""
    return BLOB_INFO_KIND

  @classmethod
  def get(cls, blob_key, **ctx_options):
    """Retrieve a BlobInfo by key.

    Args:
      blob_key: A blob key.  This may be a str, unicode or BlobKey instance.
      **ctx_options: Context options for Model().get_by_id().

    Returns:
      A BlobInfo entity associated with the provided key,  If there was
      no such entity, returns None.
    """
    fut = cls.get_async(blob_key, **ctx_options)
    return fut.get_result()

  @classmethod
  def get_async(cls, blob_key, **ctx_options):
    """Async version of get()."""
    if not isinstance(blob_key, (BlobKey, six.text_type, six.binary_type)):
      raise TypeError('Expected blob key, got %r' % (blob_key,))
    if 'parent' in ctx_options:
      raise TypeError('Parent is not supported')


    blob_key_id = str(blob_key) if isinstance(blob_key, BlobKey) else blob_key

    return cls.get_by_id_async(six.ensure_binary(blob_key_id), **ctx_options)

  @classmethod
  def get_multi(cls, blob_keys, **ctx_options):
    """Multi-key version of get().

    Args:
      blob_keys: A list of blob keys.
      **ctx_options: Context options for Model().get_by_id().

    Returns:
      A list whose items are each either a BlobInfo entity or None.
    """
    futs = cls.get_multi_async(blob_keys, **ctx_options)
    return [fut.get_result() for fut in futs]

  @classmethod
  def get_multi_async(cls, blob_keys, **ctx_options):
    """Async version of get_multi()."""
    for blob_key in blob_keys:
      if not isinstance(blob_key, (BlobKey, six.string_types)):
        raise TypeError('Expected blob key, got %r' % (blob_key,))
    if 'parent' in ctx_options:
      raise TypeError('Parent is not supported')
    blob_key_strs = list(map(str, blob_keys))
    keys = [model.Key(BLOB_INFO_KIND, id) for id in blob_key_strs]
    return model.get_multi_async(keys, **ctx_options)

  def _put_async(self, **ctx_options):
    """Cheap way to make BlobInfo entities read-only."""
    raise TypeError('BlobInfo is read-only')
  put_async = _put_async

  def key(self):
    """Get key for blob.

    Returns:
      BlobKey instance that identifies this blob.
    """

    return BlobKey(self._key.id())

  def delete(self, **options):
    """Permanently delete this blob from Blobstore.

    Args:
      **options: Options for create_rpc().
    """
    fut = delete_async(self.key(), **options)
    fut.get_result()

  def delete_async(self, **options):
    """Async version of delete()."""
    return delete_async(self.key(), **options)

  def open(self, *args, **kwds):
    """Returns a BlobReader for this blob.

    Args:
      *args, **kwargs: Passed to BlobReader constructor.

    Returns:
      A BlobReader instance.
    """
    return BlobReader(self, *args, **kwds)


get = BlobInfo.get
get_async = BlobInfo.get_async
get_multi = BlobInfo.get_multi
get_multi_async = BlobInfo.get_multi_async


def delete(blob_key, **options):
  """Delete a blob from Blobstore.

  Args:
    blob_key: A blob key.  This may be a str, unicode or BlobKey instance.
    **options: Options for create_rpc().
  """
  fut = delete_async(blob_key, **options)
  return fut.get_result()


@tasklets.tasklet
def delete_async(blob_key, **options):
  """Async version of delete()."""
  if not isinstance(blob_key, (six.string_types, BlobKey)):
    raise TypeError('Expected blob key, got %r' % (blob_key,))
  rpc = blobstore.create_rpc(**options)
  yield blobstore.delete_async(blob_key, rpc=rpc)


def delete_multi(blob_keys, **options):
  """Delete blobs from Blobstore.

  Args:
    blob_keys: A list of blob keys.
    **options: Options for create_rpc().
  """
  fut = delete_multi_async(blob_keys, **options)
  fut.get_result()


@tasklets.tasklet
def delete_multi_async(blob_keys, **options):
  """Async version of delete_multi()."""
  if isinstance(blob_keys, (six.string_types, BlobKey)):
    raise TypeError('Expected a list, got %r' % (blob_key,))
  rpc = blobstore.create_rpc(**options)
  yield blobstore.delete_async(blob_keys, rpc=rpc)


def create_upload_url(success_path,
                      max_bytes_per_blob=None,
                      max_bytes_total=None,
                      **options):
  """Create upload URL for POST form.

  Args:
    success_path: Path within application to call when POST is successful
      and upload is complete.
    max_bytes_per_blob: The maximum size in bytes that any one blob in the
      upload can be or None for no maximum size.
    max_bytes_total: The maximum size in bytes that the aggregate sizes of all
      of the blobs in the upload can be or None for no maximum size.
    **options: Options for create_rpc().

  Returns:
    The upload URL.

  Raises:
    TypeError: If max_bytes_per_blob or max_bytes_total are not integral types.
    ValueError: If max_bytes_per_blob or max_bytes_total are not
      positive values.
  """
  fut = create_upload_url_async(success_path,
                                max_bytes_per_blob=max_bytes_per_blob,
                                max_bytes_total=max_bytes_total,
                                **options)
  return fut.get_result()


@tasklets.tasklet
def create_upload_url_async(success_path,
                            max_bytes_per_blob=None,
                            max_bytes_total=None,
                            **options):
  """Async version of create_upload_url()."""
  rpc = blobstore.create_rpc(**options)
  rpc = blobstore.create_upload_url_async(success_path,
                                          max_bytes_per_blob=max_bytes_per_blob,
                                          max_bytes_total=max_bytes_total,
                                          rpc=rpc)
  result = yield rpc
  raise tasklets.Return(result)


def parse_blob_info(field_storage):
  """Parse a BlobInfo record from file upload field_storage.

  Args:
    field_storage: cgi.FieldStorage that represents uploaded blob.

  Returns:
    BlobInfo record as parsed from the field-storage instance.
    None if there was no field_storage.

  Raises:
    BlobInfoParseError when provided field_storage does not contain enough
    information to construct a BlobInfo object.
  """
  if field_storage is None:
    return None

  field_name = field_storage.name

  def get_value(dct, name):
    value = dct.get(name, None)
    if value is None:
      raise BlobInfoParseError(
          'Field %s has no %s.' % (field_name, name))
    return value

  filename = get_value(field_storage.disposition_options, 'filename')
  blob_key_str = get_value(field_storage.type_options, 'blob-key')
  blob_key = BlobKey(blob_key_str)

  if six.PY2:
    upload_content = email.message_from_file(field_storage.file)
  else:
    upload_content = email.message_from_binary_file(field_storage.file)

  content_type = get_value(upload_content, 'content-type')
  size = get_value(upload_content, 'content-length')
  creation_string = get_value(upload_content, UPLOAD_INFO_CREATION_HEADER)
  md5_hash_encoded = get_value(upload_content, 'content-md5')
  md5_hash = base64.urlsafe_b64decode(md5_hash_encoded)

  try:
    size = int(size)
  except (TypeError, ValueError):
    raise BlobInfoParseError(
        '%s is not a valid value for %s size.' % (size, field_name))

  try:
    creation = blobstore._parse_creation(creation_string, field_name)
  except blobstore._CreationFormatError as err:
    raise BlobInfoParseError(str(err))

  return BlobInfo(id=blob_key_str,
                  content_type=content_type,
                  creation=creation,
                  filename=filename,
                  size=size,
                  md5_hash=md5_hash,
                 )


def fetch_data(blob, start_index, end_index, **options):
  """Fetch data for blob.

  Fetches a fragment of a blob up to MAX_BLOB_FETCH_SIZE in length.  Attempting
  to fetch a fragment that extends beyond the boundaries of the blob will return
  the amount of data from start_index until the end of the blob, which will be
  a smaller size than requested.  Requesting a fragment which is entirely
  outside the boundaries of the blob will return empty string.  Attempting
  to fetch a negative index will raise an exception.

  Args:
    blob: BlobInfo, BlobKey, str or unicode representation of BlobKey of
      blob to fetch data from.
    start_index: Start index of blob data to fetch.  May not be negative.
    end_index: End index (inclusive) of blob data to fetch.  Must be
      >= start_index.
    **options: Options for create_rpc().

  Returns:
    str containing partial data of blob.  If the indexes are legal but outside
    the boundaries of the blob, will return empty string.

  Raises:
    TypeError if start_index or end_index are not indexes.  Also when blob
      is not a string, BlobKey or BlobInfo.
    DataIndexOutOfRangeError when start_index < 0 or end_index < start_index.
    BlobFetchSizeTooLargeError when request blob fragment is larger than
      MAX_BLOB_FETCH_SIZE.
    BlobNotFoundError when blob does not exist.
  """
  fut = fetch_data_async(blob, start_index, end_index, **options)
  return fut.get_result()


@tasklets.tasklet
def fetch_data_async(blob, start_index, end_index, **options):
  """Async version of fetch_data()."""
  if isinstance(blob, BlobInfo):
    blob = blob.key()
  rpc = blobstore.create_rpc(**options)
  rpc = blobstore.fetch_data_async(blob, start_index, end_index, rpc=rpc)
  result = yield rpc
  raise tasklets.Return(result)


class BlobReader(ext_blobstore.BlobReader):
  """Provides a read-only file-like interface to a blobstore blob."""





  def __fill_buffer(self, size=0):
    """Fills the internal buffer.

    Args:
      size: Number of bytes to read. Will be clamped to
        [self.__buffer_size, MAX_BLOB_FETCH_SIZE].
    """
    read_size = min(max(size, self.__buffer_size), MAX_BLOB_FETCH_SIZE)

    self.__buffer = fetch_data(self.__blob_key, self.__position,
                               self.__position + read_size - 1)
    self.__buffer_position = 0
    self.__eof = len(self.__buffer) < read_size

  @property
  def blob_info(self):
    """Returns the BlobInfo for this file."""
    if not self.__blob_info:
      self.__blob_info = BlobInfo.get(self.__blob_key)
    return self.__blob_info



#!/usr/bin/env python
#
# Copyright 2007 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#



"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database


_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n6google/appengine/api/blobstore/blobstore_service.proto\x12\x10google.appengine\"\xeb\x01\n\x15\x42lobstoreServiceError\"\xd1\x01\n\tErrorCode\x12\x06\n\x02OK\x10\x00\x12\x12\n\x0eINTERNAL_ERROR\x10\x01\x12\x10\n\x0cURL_TOO_LONG\x10\x02\x12\x15\n\x11PERMISSION_DENIED\x10\x03\x12\x12\n\x0e\x42LOB_NOT_FOUND\x10\x04\x12\x1b\n\x17\x44\x41TA_INDEX_OUT_OF_RANGE\x10\x05\x12\x1d\n\x19\x42LOB_FETCH_SIZE_TOO_LARGE\x10\x06\x12\x19\n\x15\x41RGUMENT_OUT_OF_RANGE\x10\x08\x12\x14\n\x10INVALID_BLOB_KEY\x10\t\"\xae\x01\n\x16\x43reateUploadURLRequest\x12\x14\n\x0csuccess_path\x18\x01 \x02(\t\x12\x1d\n\x15max_upload_size_bytes\x18\x02 \x01(\x03\x12&\n\x1emax_upload_size_per_blob_bytes\x18\x03 \x01(\x03\x12\x16\n\x0egs_bucket_name\x18\x04 \x01(\t\x12\x1f\n\x17url_expiry_time_seconds\x18\x05 \x01(\x05\"&\n\x17\x43reateUploadURLResponse\x12\x0b\n\x03url\x18\x01 \x02(\t\"4\n\x11\x44\x65leteBlobRequest\x12\x10\n\x08\x62lob_key\x18\x01 \x03(\t\x12\r\n\x05token\x18\x02 \x01(\t\"L\n\x10\x46\x65tchDataRequest\x12\x10\n\x08\x62lob_key\x18\x01 \x02(\t\x12\x13\n\x0bstart_index\x18\x02 \x02(\x03\x12\x11\n\tend_index\x18\x03 \x02(\x03\"&\n\x11\x46\x65tchDataResponse\x12\x11\n\x04\x64\x61ta\x18\xe8\x07 \x02(\x0c\x42\x02\x08\x01\"N\n\x10\x43loneBlobRequest\x12\x10\n\x08\x62lob_key\x18\x01 \x02(\x0c\x12\x11\n\tmime_type\x18\x02 \x02(\x0c\x12\x15\n\rtarget_app_id\x18\x03 \x02(\x0c\"%\n\x11\x43loneBlobResponse\x12\x10\n\x08\x62lob_key\x18\x01 \x02(\x0c\"(\n\x14\x44\x65\x63odeBlobKeyRequest\x12\x10\n\x08\x62lob_key\x18\x01 \x03(\t\"(\n\x15\x44\x65\x63odeBlobKeyResponse\x12\x0f\n\x07\x64\x65\x63oded\x18\x01 \x03(\t\"8\n$CreateEncodedGoogleStorageKeyRequest\x12\x10\n\x08\x66ilename\x18\x01 \x02(\t\"9\n%CreateEncodedGoogleStorageKeyResponse\x12\x10\n\x08\x62lob_key\x18\x01 \x02(\tB8\n\"com.google.appengine.api.blobstoreB\x12\x42lobstoreServicePb')



_BLOBSTORESERVICEERROR = DESCRIPTOR.message_types_by_name['BlobstoreServiceError']
_CREATEUPLOADURLREQUEST = DESCRIPTOR.message_types_by_name['CreateUploadURLRequest']
_CREATEUPLOADURLRESPONSE = DESCRIPTOR.message_types_by_name['CreateUploadURLResponse']
_DELETEBLOBREQUEST = DESCRIPTOR.message_types_by_name['DeleteBlobRequest']
_FETCHDATAREQUEST = DESCRIPTOR.message_types_by_name['FetchDataRequest']
_FETCHDATARESPONSE = DESCRIPTOR.message_types_by_name['FetchDataResponse']
_CLONEBLOBREQUEST = DESCRIPTOR.message_types_by_name['CloneBlobRequest']
_CLONEBLOBRESPONSE = DESCRIPTOR.message_types_by_name['CloneBlobResponse']
_DECODEBLOBKEYREQUEST = DESCRIPTOR.message_types_by_name['DecodeBlobKeyRequest']
_DECODEBLOBKEYRESPONSE = DESCRIPTOR.message_types_by_name['DecodeBlobKeyResponse']
_CREATEENCODEDGOOGLESTORAGEKEYREQUEST = DESCRIPTOR.message_types_by_name['CreateEncodedGoogleStorageKeyRequest']
_CREATEENCODEDGOOGLESTORAGEKEYRESPONSE = DESCRIPTOR.message_types_by_name['CreateEncodedGoogleStorageKeyResponse']
_BLOBSTORESERVICEERROR_ERRORCODE = _BLOBSTORESERVICEERROR.enum_types_by_name['ErrorCode']
BlobstoreServiceError = _reflection.GeneratedProtocolMessageType('BlobstoreServiceError', (_message.Message,), {
  'DESCRIPTOR' : _BLOBSTORESERVICEERROR,
  '__module__' : 'google.appengine.api.blobstore.blobstore_service_pb2'

  })
_sym_db.RegisterMessage(BlobstoreServiceError)

CreateUploadURLRequest = _reflection.GeneratedProtocolMessageType('CreateUploadURLRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEUPLOADURLREQUEST,
  '__module__' : 'google.appengine.api.blobstore.blobstore_service_pb2'

  })
_sym_db.RegisterMessage(CreateUploadURLRequest)

CreateUploadURLResponse = _reflection.GeneratedProtocolMessageType('CreateUploadURLResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATEUPLOADURLRESPONSE,
  '__module__' : 'google.appengine.api.blobstore.blobstore_service_pb2'

  })
_sym_db.RegisterMessage(CreateUploadURLResponse)

DeleteBlobRequest = _reflection.GeneratedProtocolMessageType('DeleteBlobRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEBLOBREQUEST,
  '__module__' : 'google.appengine.api.blobstore.blobstore_service_pb2'

  })
_sym_db.RegisterMessage(DeleteBlobRequest)

FetchDataRequest = _reflection.GeneratedProtocolMessageType('FetchDataRequest', (_message.Message,), {
  'DESCRIPTOR' : _FETCHDATAREQUEST,
  '__module__' : 'google.appengine.api.blobstore.blobstore_service_pb2'

  })
_sym_db.RegisterMessage(FetchDataRequest)

FetchDataResponse = _reflection.GeneratedProtocolMessageType('FetchDataResponse', (_message.Message,), {
  'DESCRIPTOR' : _FETCHDATARESPONSE,
  '__module__' : 'google.appengine.api.blobstore.blobstore_service_pb2'

  })
_sym_db.RegisterMessage(FetchDataResponse)

CloneBlobRequest = _reflection.GeneratedProtocolMessageType('CloneBlobRequest', (_message.Message,), {
  'DESCRIPTOR' : _CLONEBLOBREQUEST,
  '__module__' : 'google.appengine.api.blobstore.blobstore_service_pb2'

  })
_sym_db.RegisterMessage(CloneBlobRequest)

CloneBlobResponse = _reflection.GeneratedProtocolMessageType('CloneBlobResponse', (_message.Message,), {
  'DESCRIPTOR' : _CLONEBLOBRESPONSE,
  '__module__' : 'google.appengine.api.blobstore.blobstore_service_pb2'

  })
_sym_db.RegisterMessage(CloneBlobResponse)

DecodeBlobKeyRequest = _reflection.GeneratedProtocolMessageType('DecodeBlobKeyRequest', (_message.Message,), {
  'DESCRIPTOR' : _DECODEBLOBKEYREQUEST,
  '__module__' : 'google.appengine.api.blobstore.blobstore_service_pb2'

  })
_sym_db.RegisterMessage(DecodeBlobKeyRequest)

DecodeBlobKeyResponse = _reflection.GeneratedProtocolMessageType('DecodeBlobKeyResponse', (_message.Message,), {
  'DESCRIPTOR' : _DECODEBLOBKEYRESPONSE,
  '__module__' : 'google.appengine.api.blobstore.blobstore_service_pb2'

  })
_sym_db.RegisterMessage(DecodeBlobKeyResponse)

CreateEncodedGoogleStorageKeyRequest = _reflection.GeneratedProtocolMessageType('CreateEncodedGoogleStorageKeyRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEENCODEDGOOGLESTORAGEKEYREQUEST,
  '__module__' : 'google.appengine.api.blobstore.blobstore_service_pb2'

  })
_sym_db.RegisterMessage(CreateEncodedGoogleStorageKeyRequest)

CreateEncodedGoogleStorageKeyResponse = _reflection.GeneratedProtocolMessageType('CreateEncodedGoogleStorageKeyResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATEENCODEDGOOGLESTORAGEKEYRESPONSE,
  '__module__' : 'google.appengine.api.blobstore.blobstore_service_pb2'

  })
_sym_db.RegisterMessage(CreateEncodedGoogleStorageKeyResponse)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\"com.google.appengine.api.blobstoreB\022BlobstoreServicePb'
  _FETCHDATARESPONSE.fields_by_name['data']._options = None
  _FETCHDATARESPONSE.fields_by_name['data']._serialized_options = b'\010\001'
  _BLOBSTORESERVICEERROR._serialized_start=77
  _BLOBSTORESERVICEERROR._serialized_end=312
  _BLOBSTORESERVICEERROR_ERRORCODE._serialized_start=103
  _BLOBSTORESERVICEERROR_ERRORCODE._serialized_end=312
  _CREATEUPLOADURLREQUEST._serialized_start=315
  _CREATEUPLOADURLREQUEST._serialized_end=489
  _CREATEUPLOADURLRESPONSE._serialized_start=491
  _CREATEUPLOADURLRESPONSE._serialized_end=529
  _DELETEBLOBREQUEST._serialized_start=531
  _DELETEBLOBREQUEST._serialized_end=583
  _FETCHDATAREQUEST._serialized_start=585
  _FETCHDATAREQUEST._serialized_end=661
  _FETCHDATARESPONSE._serialized_start=663
  _FETCHDATARESPONSE._serialized_end=701
  _CLONEBLOBREQUEST._serialized_start=703
  _CLONEBLOBREQUEST._serialized_end=781
  _CLONEBLOBRESPONSE._serialized_start=783
  _CLONEBLOBRESPONSE._serialized_end=820
  _DECODEBLOBKEYREQUEST._serialized_start=822
  _DECODEBLOBKEYREQUEST._serialized_end=862
  _DECODEBLOBKEYRESPONSE._serialized_start=864
  _DECODEBLOBKEYRESPONSE._serialized_end=904
  _CREATEENCODEDGOOGLESTORAGEKEYREQUEST._serialized_start=906
  _CREATEENCODEDGOOGLESTORAGEKEYREQUEST._serialized_end=962
  _CREATEENCODEDGOOGLESTORAGEKEYRESPONSE._serialized_start=964
  _CREATEENCODEDGOOGLESTORAGEKEYRESPONSE._serialized_end=1021

